package com.brainysoftware.jdk5.app25e;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.GenericServlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

public class PageCounterServlet extends GenericServlet {
    private static final String PAGE_COUNTER_FILE_PATH = "WEB-INF/pageCounter.txt";

    private static final String PAGE_COUNTER_ATTRIBUTE_NAME = "pageCounter";

    public void init() throws ServletException {
        long pageCounter = 0;
        ServletContext servletContext = getServletContext();
        String realPath = servletContext.getRealPath(PAGE_COUNTER_FILE_PATH);
        File file = new File(realPath);
        if (file.exists()) {
            // open and read page counter value from file
            try {
                FileReader fr = new FileReader(file);
                BufferedReader br = new BufferedReader(fr);
                pageCounter = Long.parseLong(br.readLine());
                br.close();
                fr.close();
            } catch (IOException e) {
            } catch (NumberFormatException e) {
            }
        }
        servletContext.setAttribute(PAGE_COUNTER_ATTRIBUTE_NAME, pageCounter);
    }

    public void service(ServletRequest request, ServletResponse response)
            throws ServletException, IOException {
        incrementPageCounter();
        response.setContentType("text/html");
        PrintWriter writer = response.getWriter();
        writer.println("<html>");
        writer.println("<head><title>Page Counter</title></head");
        writer.println("<body>");
        writer.println("You are visitor number " + readPageCounter());
        writer.println("</body>");
        writer.println("</html>");
    }

    public void destroy() {
        // save the counter value to text file
        ServletContext servletContext = getServletContext();
        String realPath = servletContext.getRealPath(PAGE_COUNTER_FILE_PATH);
        File file = new File(realPath);
        try {
            PrintWriter pw = new PrintWriter(file);
            pw.write(Long.toString(readPageCounter()));
            pw.close();
        } catch (IOException e) {
        }
    }

    private void incrementPageCounter() {
        ServletContext servletContext = getServletContext();
        // no need to catch NumberFormatException
        Long pageCounter = (Long) servletContext
                .getAttribute(PAGE_COUNTER_ATTRIBUTE_NAME);
        synchronized (pageCounter) {
            pageCounter++;
            servletContext.setAttribute(PAGE_COUNTER_ATTRIBUTE_NAME,
                    pageCounter);
        }
    }

    private long readPageCounter() {
        long returnValue = 0;
        ServletContext servletContext = getServletContext();
        Long pageCounter = (Long) servletContext
                .getAttribute(PAGE_COUNTER_ATTRIBUTE_NAME);
        synchronized (pageCounter) {
            returnValue = pageCounter;
        }
        return returnValue;
    }
}
